#include <xbee.h>
#include <SoftwareSerial.h>
// #define PIN_IRLED	14			// アナログ入力0=デジタル14(ir_send.inoに記載)
#define PIN_LUX		1				// アナログ入力1
// #define PIN_IRIN	16				// アナログ入力2=デジタル16
#define PIN_LED		13				// 動作確認用
#define AEHA		0
#define NEC			1
#define SIRC		2
#define RX_MAX		64
#define TX_MAX		32				// TX_MAX <= RX_MAX
#define IR_LEN		21				// IR信号の長さ＋5バイト（ヘッダ用）

SoftwareSerial AquesSerial(2,3);	// 終端文字は \r CR 0x0D
SoftwareSerial LedSerial(4,5);		// 終端改行は \n LF 0x0A
byte SoftwareSerial_RxPort=0;		// ソフトシリアルは同時に待ち受けが出来ない

byte dev[]={0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}; // オール0はCoordinator
byte ir[IR_LEN];
byte len=0;									// IR信号長
byte type = AEHA;							// 赤外線方式

void setup(){
	byte i=0,j=0;
	analogReference(DEFAULT);		// ADC(照度入力)のリファレンス電圧(3.3V)
//	analogReference(INTERNAL);		// ADC(照度入力)のリファレンス電圧(1.1V)
	pinMode(PIN_LED, OUTPUT);			// 動作確認用LED
	digitalWrite(PIN_LED, HIGH);
	ir_read_init();					// IR受信センサ
	ir_init();						// IR送信ダイオード
	xbee_init( 0x00 );				// XBee用COMポートの初期化
	xbee_atnj( 0x00 );				// ジョイン不許可
	LedSerial.begin(9600);
	AquesSerial.begin(9600);
	AquesSerial.print("#J\r");
	delay(1000);
	LedSerial.println("XBee Started");
	delay(500);
	AquesSerial.print("?konnitiwa'\r");
	delay(2000);
	digitalWrite(13, LOW);
	while( xbee_ping(dev) == 0xFF ){            // ping応答を確認
		while( xbee_atai() > 0x01 ){            // ネットワーク参加状況を確認
			AquesSerial.print("?kensakutyuudesu'\r");
			for(i=0;i<60;i++){
				digitalWrite(PIN_LED,i%2);      // LEDの高速点滅
				delay(30);						// 1800ms
			}
			AquesSerial.print("#K\r");
			xbee_atcb(1);                       // ネットワーク参加ボタンを押下
			j++;
			if(j>10){
				j=0;
				xbee_atcb(4);                   // ネットワーク初期化
			}
			delay(1000);
		}
		delay(1000);
	}
	LedSerial.println("Connected");
	delay(500);
	AquesSerial.print("?setuzokusima'sita\r");
	delay(2000);
	if(i==0)xbee_atcb(1);
}

void loop(){
	byte i;
	int j;
	byte data;
	char c;
	char s[RX_MAX];
	unsigned int value_i;
	
	XBEE_RESULT xbee_result;					// 受信データ用の構造体
	
	// メイン処理
	LedSerial.listen();
	
	while(1){									// 永久に受信する
		i=1;									// ソフトウェアシリアル受信フラグ
		s[0]=0x1B;
		if( LedSerial.isListening() ){
			/* マトリクスLEDからの受信 */
			while(LedSerial.available() && i<(RX_MAX-1)){
				c=LedSerial.read();
				if( c !='\0' && c !='\n' && c!='\r' ){
					s[i]=c;
					i++;
					s[i]='\0';
				}
				delay(1);
			}
		}
		if(i>1 && s[1] !='>' ){
			if( s[1] =='2' && s[2] =='2'  && isdigit(s[3]) ){
				s[1]='E'; s[2]='N';
				i=0;
			}else if(isdigit(s[1]) && isdigit(s[2]) && s[3]==':' && isdigit(s[4]) &&isdigit(s[5]) ){
				i=0;							// 時刻をLEDから受信
				s[6]='\0';
			}
			if(s[1] =='T' && (s[5] !='0'  || s[6] !='0') ) i=0;
			if(s[1] =='E' && s[2] =='R'  && s[3] =='R' ) i=0;
			if(i==0) xbee_uart(dev,s);
		}
		
		/* 赤外線リモコンの受信 */
		ir_delay(0);
	
		/* XBeeからの受信(待ち受けて受信する) */
		data = xbee_rx_call( &xbee_result );

		switch( xbee_result.MODE ){				// 受信したデータの内容(MODE値)に応じて
			case MODE_UART:
				if(xbee_result.DATA[0] == (byte)'?' ){	// TALKコマンド
					if(xbee_result.DATA[1]=='#'){
						AquesSerial.print("#J\r");
						ir_delay(1000);
					}else if(data < RX_MAX-1){
						memset(s,0x00,sizeof(s));
						for(i=0;i<data;i++){
							if( xbee_result.DATA[i] >= 0x1B ){
								s[i]=(char)xbee_result.DATA[i];
							}else break;
						}
						s[i]='\r';	// Aques Talk改行コード
						s[i+1]='\0';
						AquesSerial.print(s);
						ir_delay(2000);
					}
				}else if(xbee_result.DATA[0] == 0x1B){    // ESCコードが先頭だったとき
					if(xbee_result.DATA[1]==(byte)'I'){
						if( xbee_result.DATA[2]==(byte)'R'){    // コマンドが「IR」の時
							type = xbee_result.DATA[3];
							len = xbee_result.DATA[4];
							if( len+5 <= data){
								ir_send( &xbee_result.DATA[5] , len , type );
						//		ir_delay(2000);
							}else{
								LedSerial.println("IR ERROR");
								ir_delay(500);
								AquesSerial.print("#K\r");
								ir_delay(1000);
								AquesSerial.print("?e'ra-desu\r");	// エラーです
								ir_delay(2000);
							}
						}else if( xbee_result.DATA[2]==(byte)'S'){    // コマンドが「IS」の時
							i=(byte)getTemp();
							j=analogRead(PIN_LUX);
							sprintf(s,"%d,%d",j,i);
							xbee_uart(dev,s);
						}
					}else{	// その他のESCコード　（?やI以外）
				/*		case (byte)'T':	// TIME
						case (byte)'F':	// FAN
						case (byte)'L':	// LEVEL
						default:	*/
						/* LEDへ転送 */
						memset(s,0x00,sizeof(s));
						if(data < RX_MAX-1){
							for(i=0;i<data;i++) s[i]=(char)xbee_result.DATA[i];
							s[RX_MAX-1]='\0';	// 保険
							if(s[data-1]=='\n' || s[data-1]=='\r') s[data-1]='\0';
							if(s[data-2]=='\r') s[data-2]='\0';
							LedSerial.listen();
							LedSerial.println(s);
							ir_delay(500);
						}
						if(xbee_result.DATA[1]==(byte)'T'){
							AquesSerial.print("#J\r");
							ir_delay(1000);
							sprintf(s,"?<NUMK VAL=%c%c COUTER=ji>desu\r",s[2],s[3]);
							AquesSerial.print(s);
							ir_delay(2000);
						}else if(xbee_result.DATA[1]==(byte)'F'){
							sprintf(s,"?<NUMK VAL=%c%c>donisette-\r",s[2],s[3]);
							AquesSerial.print(s);
							ir_delay(2000);
						}
						ir_delay(1000);
					}
				}else{	// その他のシリアル情報 (テキストのみ)
					if(data < RX_MAX-1 && data > 0){
						memset(s,0x00,sizeof(s));
						for(i=0;i<data;i++) s[i]=(char)xbee_result.DATA[i];
						LedSerial.listen();
						LedSerial.println(s);
						ir_delay(500);
					}
				}
				break;
			case MODE_GPIN:
				sprintf(s,"GPIN=%02X",data);
				LedSerial.listen();
				LedSerial.println(s);
				ir_delay(500);
				break;
			case MODE_IDNT:
				sprintf(s,"IDNT=");
				for(i=0;i<8;i++) sprintf(s,"%s%02X",s,xbee_result.FROM[i]);
				LedSerial.listen();
				LedSerial.println(s);
				ir_delay(500);
			//	AquesSerial.print("?da'rekaga/sankasi'masita\r");	// 誰かが参加しました
				AquesSerial.print("#J\r");	// 誰かが参加しました
				ir_delay(1000);
				break;
			default:
				break;
		}
	}
}

void ir_delay(unsigned int t){
	t /= 50;
	do{
		len = (byte)(abs(ir_read(&ir[5], IR_LEN-5, type)) / 8);	// -1,-2,-3値は0になる
		if(len>=4 && len <= IR_LEN-5 ){
			ir[0]=0x1B;	ir[1]='I';	ir[2]='R';	ir[3]=type;	ir[4]=len;
			xbee_bytes(dev,ir,len+5);
		}
		if(t>0){
			xbee_delay(50);
			t--;
		}
	}while(t>0);
}

/* データ例
 ESC  'I'  'R'  type len  data....
{0x1B,0x49,0x52,0x00,0x06,0xAA,0x5A,0x8F,0x12,0x15,0xE1}    // AQUOS 音量を下げる	1B 49 52 00 06 AA 5A 8F 12 15 E1 
{0x1B,0x49,0x52,0x00,0x05,0x2C,0x52,0x09,0x2B,0x22};        // 照明の輝度を下げる

*/
