/*********************************************************************
EEPROMへフォントを書き込みます。

下記のサイトに詳細情報を掲載しています。

http://www.geocities.jp/bokunimowakaru/diy/arduino/matrix_led.html

本ソースリストおよびソフトウェアは、ライセンスフリーです。
利用、編集、再配布等が自由に行えますが、著作権表示の改変は禁止します。

                               Copyright (c) 2014 Wataru KUNINO
                               http://www.geocities.jp/bokunimowakaru/

※注意
	ライブラリ「FrequencyTimer2」と「Time」を使用します。
	同ライブラリの制約上、Leonardoでは動作いたしません。
	
	動作が安定しない場合は、font.ino の「#define EEPROM_WRITE」の行を
	削除してメモリを確保してください。EEPROM_WRITEはEEPROMへフォントを
	書き込むスイッチです。但し、必ず、一度だけはEEPROM_WRITEを有効に
	して実行してください。（フォントをマイコンに書き込む必要があります。）
	
*********************************************************************/
#include "AkiMatrix.h"
#include <FrequencyTimer2.h>
#include <EEPROM.h>
#include <Time.h>

#define	DISPLAY_COLS	224		// 表示用のバッファサイズ 最大256-32
#define	LED_COMMON		1		// LEDの極性。カソードがROW側(Aタイプ)は0、COL側(Bタイプ)は1
#define	RX_MAX			64		// シリアル受信バッファサイズ
#define	LED_DURATION	50		// LEDの点灯時間(最大255ms)[ms]
#define PIN_COOLFAN		13		// 冷却ファンのピン番号
byte	TEMP_COOLFAN=	35	;	// 冷却ファンの動作開始温度
#define	TEMP_OVERHEAT	40		// オーバーヒート（表示オフ）

AkiMatrix myMatrix = AkiMatrix(2,3,4,5,6,7);
#define	PIN_STROB		7		// STROB端子 AkiMatrixの第6引数

////////////////////////////////////////////////////////////////////////////////////////////////

/* シリアルからの文字入力用の変数定義 */
char rx[RX_MAX+1];				// 表示テキスト入力データ保持用(ここにテキスト文字を代入する)
byte rx_i;						// シリアル受信したテキスト文字の位置番号

/* マトリクスLEDの駆動用の変数定義 */
byte ledc[28];					// 上行に常時表示する画素データ
byte leds[DISPLAY_COLS+4];		// 下行に表示する画素データ保持用
boolean text_disp	= 1;		// テキスト表示
byte pattern 		= 0;		// スクロール表示のまめの表示位置指定用の変数
byte pattern_max 	= 0;		// スクロール表示の最終桁

/* 時計用の変数定義 */
boolean esc_enable	= 0;		// ESCコード状態
boolean clock_disp	= 0;		// 時計表示
byte clock_sec;					// 前回秒
byte clock_min;					// 前回分
char clock_str[7];				// 時刻テキスト

/* レベルメータ表示 */
byte level1=0;					// レベルメータ表示
byte level2=0;					// レベルメータ表示

boolean coolfan		= 0;		// 冷却ファンの状態
boolean led_on		= 1;		// LEDの表示ON
extern boolean font_kana_mode;	// fontライブラリのグローバル変数(カナモード)

void setup() {
	pinMode(PIN_STROB,OUTPUT);
	digitalWrite(PIN_STROB,1);	// OFF
	pinMode(PIN_COOLFAN,OUTPUT);

	Serial.begin(9600);
	font_kanji_init();
	clearTextPattern();
	sprintf(rx,"{ﾎﾞｸ}ﾆﾓﾜｶﾙ{FD}{9E}{ﾃﾞｼﾞﾀﾙ}{89}{9B}");
	utf_del_uni(rx);
	setTextPattern(rx);

	myMatrix.setOutputEnable(true);
	esc_enable=0;
	rx_i=0;
	Serial.println("STARTED");
}

void loop() {
	byte x,y,i;
	char esc[8];
	char c;

	pattern = ++pattern;							// LEDスクロール位置変更
	if(rx_i==RX_MAX){								// 改行もしくは受信文字列オーバを受信したら
		if(esc_enable){								// ESCコード
			switch(esc[0]){
				case 'T':	// 時刻設定
					if( isdigit(esc[1]) ){			// [ESC][T]の後に数字が続くとき
						clock_adj( &esc[1] );		// 時刻調整を実施
						clock(clock_str);
					}
					clock_min=0xFF;					// 前回分を255に設定することで即表示
					clock_disp=1;
					Serial.println(clock_str);
					break;
				case 'F':	// ファン開始温度設定
					if( isdigit(esc[1]) ){				// オフ
						TEMP_COOLFAN=(byte)(esc[1]-'0');
						if( isdigit(esc[2]) && rx_i>2 ){
							TEMP_COOLFAN*=10;
							TEMP_COOLFAN+=(byte)(esc[2]-'0');
						}
					}
					if(TEMP_COOLFAN>TEMP_OVERHEAT)TEMP_COOLFAN=TEMP_OVERHEAT;
					i=(byte)getTemp();
					if(i>99) i=99;
					sprintf(esc,"%2d,%2d",TEMP_COOLFAN,i);
					Serial.println(esc);
					break;
				case 'L':	// レベルメータ
					if( esc[1] >= '0' && esc[1] <= '8' ){
						level1 = (byte)esc[1] - (byte)'0';
						if( esc[2] >= '0' && esc[2] <= '8' ){
							level2 = (byte)esc[2] - (byte)'0';
						}
					}
					setLevelPattern();				// レベルメータのパターンをセット
					sprintf(esc,"%01X,%01X",level1,level2);
					Serial.println(esc);
					break;
				case 'P':						// パワーセーブ 両方オフで0.7W
					switch( esc[1] ){
						case '0':						// 解除	3.4 W
							text_disp=1;
							break;
						case '1':						// テキストオフ	1.7 W
							text_disp=0;
							for(x=0;x<32;x++) for(y=8;y<16;y++) myMatrix.write(x,y,0);
							break;
						case '2':						// クロックオフ 2.2 W
							clock_disp=0;
							for(x=0;x<32;x++) for(y=0;y<8;y++) myMatrix.write(x,y,0);
							break;
						default:
							break;
					}
					sprintf(esc,"%01X,%01X",(byte)text_disp,(byte)clock_disp);
					Serial.println(esc);
					break;
				default:
					Serial.println("ERR");
					break;
			}
			esc_enable=0;
			rx_i=0;										// 次の受信に備える
		}else if(pattern >= pattern_max ){				// 最後まで表示したら
			clearTextPattern();							// LED表示用のピクセルバッファを消去
			utf_del_uni(rx);
			setTextPattern(rx);							// 受信した文字をセット
			Serial.println(pattern_max);				// パターン長(最大255)
			rx_i=0;										// 次の受信に備える
			pattern=0;									// 最初に戻す
		}
	}
	if(pattern >= pattern_max ) pattern=0;				// 最初に戻す
	/* テキスト表示 */
	if(text_disp){
		for(i=0;(i+pattern)<32;i++) for(y=8;y<16;y++) myMatrix.write(i,y,0);
		if(pattern<32) i=32-pattern; else i=0;
		for(y=8;y<16;y++){
			for(x=i;x<32;x++){
				myMatrix.write(x,y,(leds[x+pattern-32]>>(y-8))&1);
			}
		}
	}
	/* 時計表示 */
	if( clock_sec != second() ){						// 1秒ごとに以下を実行
		clock_sec = second();
		if( clock_disp ){
			for(y=1;y<=5;y++){	// コロンの点滅用
				if(y==3)y++;
				for(x=13;x<=14;x++){
					myMatrix.write(x,y,(clock_sec%2));
				}
			}
		}									// 1分ごとに以下を実行
		if( clock_min != minute() ){
			clock_min = minute();
			if( clock_disp ){
				clock(clock_str);						// 時計表示データを作成
				setClockPattern(clock_str);
				for(y=0;y<8;y++){
					for(x=0;x<28;x++){
						myMatrix.write(x,y,(ledc[x]>>y)&1);
					}
				}
			}
			i=(byte)getTemp();
			if( i > TEMP_COOLFAN ) coolfan=1;
			if( i < TEMP_COOLFAN ) coolfan=0;
			digitalWrite(PIN_COOLFAN,coolfan);				// 冷却ファン
			if( i >= TEMP_OVERHEAT ){
				for(x=0;x<32;x++) for(y=0;y<16;y++) myMatrix.write(x,y,0);
				text_disp=0;
				clock_disp=0;
			}
		}
	}
	delay(LED_DURATION);							// スクロール用のウェイト

	/*シリアルからの受信処理*/
	if (Serial.available() && rx_i < RX_MAX){
		c=Serial.read();							// 受信値をバッファへ保存
		if( c==0x1B && rx_i==0) esc_enable=1;		// ESCコードの設定
		else{
			if(esc_enable){
				esc[rx_i]=c;
				rx_i++;
				if(rx_i>=7){
					rx_i=RX_MAX;
					rx[rx_i]='\0';
				}else if(c=='\n'){
					esc[rx_i]='\0';						// シリアルバッファに終了文字を入れる
					rx_i=RX_MAX;						// これ以上、受信しない
				}
			}else{
				rx[rx_i]=c;
				if( isprint(c) || (byte)c > 0x80){	// 表示可能な文字の時
					rx_i++;
					if(rx_i>RX_MAX){
						rx_i=RX_MAX;
						rx[rx_i]='\0';
					}
				}else if(c=='\n'){
					rx[rx_i]='\0';						// シリアルバッファに終了文字を入れる
					rx_i=RX_MAX;						// これ以上、受信しない
				}
			}
		}
	}
}

void clearTextPattern() {
	FrequencyTimer2::disable();								// Turn off toggling of pin 11
	for(int i=0; i <= DISPLAY_COLS; i++) leds[i] = 0x00;	// Clear display array
}

void setTextPattern(char *s) {
	int i=0;
	pattern=0;
	
	while(s[i] != '\0' && pattern < DISPLAY_COLS-32 ){
		pattern += font_draw(leds,s[i],pattern);
		i++;
	}
	font_kana_mode=0;										// カナモードの終端が無かった場合の対策
	pattern_max = pattern+32;
	pattern=0;
	FrequencyTimer2::setPeriod(1000);						// Set refresh rate
	FrequencyTimer2::setOnOverflow(display);				// Set interrupt routine to be called
}

void setClockPattern(char *s) {
	int i,j;
	
	j = font_draw(ledc,s[0],0);
	j += font_draw(ledc,s[1],j);
	j = 13 -j;
	for(i=0;i<28;i++) ledc[i] = 0x00;						// Clear display array
	i=0;
	while(s[i] != '\0' && j<24){
		j += font_draw(ledc,s[i],j);
		i++;
	}
	font_kana_mode=0;										// カナモードの終端が無かった場合の対策
}

void setLevelPattern(void){
	byte i;
	for(i=1;i<=8;i++) if( i <= level1 ){
		myMatrix.write(30,8-i,1); 
	}else{
		myMatrix.write(30,8-i,0);
	}
	for(i=1;i<=8;i++) if( i <= level2 ){
		myMatrix.write(31,8-i,1);
	}else{
		myMatrix.write(31,8-i,0);
	}
}

// Interrupt routine
void display() {
	myMatrix.hsync();
}

void clock(char *s){
	byte hh = (byte)hour();
	byte mm = (byte)minute();
	s[0] = (char)( hh/10 )+'0';
	s[1] = (char)( hh%10 )+'0';
	s[3] = (char)( mm/10 )+'0';
	s[4] = (char)( mm%10 )+'0';
	s[2] = ':';
	s[5] = '\0';
}

void clock_adj(char *s){
	if( s[0] >= '0' && s[0] <= '9' &&
		s[1] >= '0' && s[1] <= '9' &&
		s[2] == ':' &&
		s[3] >= '0' && s[3] <= '9' &&
		s[4] >= '0' && s[4] <= '9'){
		setTime(	(int)(	 (s[0]-'0')*10
							+(s[1]-'0')		),
					(int)(	 (s[3]-'0')*10
							+(s[4]-'0')		),0,NULL,NULL,NULL);
	}
}
